'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = undefined;

var _typeof2 = require('babel-runtime/helpers/typeof');

var _typeof3 = _interopRequireDefault(_typeof2);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _d = require('d3');

var d3 = _interopRequireWildcard(_d);

var _default = require('./default');

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var Simulation = function () {
  function Simulation(options) {
    var _this = this;

    (0, _classCallCheck3.default)(this, Simulation);
    this.sourceKey = 'source';
    this.targetKey = 'target';
    this.width = _default.WIDTH;
    this.height = _default.HEIGHT;
    this.nodeIdKey = _default.NODE_ID_KEY;
    this.nodes = [];
    this.links = [];
    this.nodesMap = {};
    this.translate = [0, 0];
    this.scale = 1;

    this.tick = function (event) {
      _this.simulation.on('tick', function () {
        event.tick && event.tick(_this.simulation.alpha());
      }).on('end', function () {
        event.end && event.end(_this.simulation.alpha());
      });
    };

    var simulation = d3.forceSimulation();
    this.simulation = simulation;
    this.setSimulationLayout(options);
  }

  (0, _createClass3.default)(Simulation, [{
    key: 'setSimulationLayout',
    value: function setSimulationLayout(_ref) {
      var velocityDecay = _ref.velocityDecay,
          nodeIdKey = _ref.nodeIdKey,
          linkDistance = _ref.linkDistance,
          collideRadius = _ref.collideRadius,
          collideStrength = _ref.collideStrength,
          chargeStrength = _ref.chargeStrength,
          alphaDecay = _ref.alphaDecay,
          alphaMin = _ref.alphaMin,
          XYCenter = _ref.XYCenter,
          width = _ref.width,
          height = _ref.height,
          sourceKey = _ref.sourceKey,
          targetKey = _ref.targetKey;

      this.width = width;
      this.height = height;
      this.nodeIdKey = nodeIdKey;
      if (sourceKey) {
        this.sourceKey = sourceKey;
      }
      if (targetKey) {
        this.targetKey = targetKey;
      }
      if (velocityDecay) {
        this.simulation.velocityDecay(velocityDecay);
      }
      if (alphaMin) {
        this.simulation.alphaMin(alphaMin);
      }
      if (alphaDecay) {
        this.simulation.alphaDecay(alphaDecay);
      }
      var forceLink = d3.forceLink();
      if (nodeIdKey) {
        forceLink.id(function (d) {
          return d[nodeIdKey];
        });
      }
      if (linkDistance) {
        forceLink.distance(linkDistance);
      }
      this.simulation.force('link', forceLink);
      this.simulation.force('center', d3.forceCenter());

      if (collideRadius || collideStrength) {
        var forceCollide = d3.forceCollide();
        if (collideRadius) {
          forceCollide.radius(typeof collideRadius === 'function' ? function (d) {
            return collideRadius(d);
          } : collideRadius);
        }
        if (collideStrength) {
          forceCollide.strength(collideStrength);
        }
        this.simulation.force('collide', forceCollide);
      } else {
        this.simulation.force('collide', null);
      }

      if (chargeStrength) {
        this.simulation.force("charge", d3.forceManyBody().strength(chargeStrength));
      }
      if (XYCenter) {
        this.simulation.force('x', d3.forceX(XYCenter && XYCenter.x || undefined));
        this.simulation.force('y', d3.forceY(XYCenter && XYCenter.y || undefined));
      } else {
        this.simulation.force('x', null);
        this.simulation.force('y', null);
      }
    }
  }, {
    key: 'initNodes',
    value: function initNodes(nodes) {
      var _this2 = this;

      var nodeIdKey = this.nodeIdKey,
          width = this.width,
          height = this.height;

      var nodesMap = {};
      var newNodes = [];
      nodes.forEach(function (node) {
        node.x = node.x || width / 2;
        node.y = node.y || height / 2;
        if (!_this2.nodesMap[node[nodeIdKey]]) {
          nodesMap[node[nodeIdKey]] = node;
          newNodes.push(node);
        } else {
          nodesMap[node[nodeIdKey]] = _this2.nodesMap[node[nodeIdKey]];
          newNodes.push(_this2.nodesMap[node[nodeIdKey]]);
        }
      });
      this.nodesMap = nodesMap;
      this.nodes = newNodes;
    }
  }, {
    key: 'initLinks',
    value: function initLinks(links) {
      var _this3 = this;

      var nodeIdKey = this.nodeIdKey,
          sourceKey = this.sourceKey,
          targetKey = this.targetKey;

      var newLinks = [];
      links.forEach(function (link) {
        var source = link[sourceKey],
            target = link[targetKey];
        if ((typeof source === 'undefined' ? 'undefined' : (0, _typeof3.default)(source)) === 'object') {
          source = source[nodeIdKey];
        }
        if ((typeof target === 'undefined' ? 'undefined' : (0, _typeof3.default)(target)) === 'object') {
          target = target[nodeIdKey];
        }
        var sourcePush = _this3.nodesMap[source];
        var targetPush = _this3.nodesMap[target];
        if (sourcePush && targetPush) {
          newLinks.push({ source: sourcePush, target: targetPush });
        }
      });
      this.links = newLinks;
    }
  }, {
    key: 'setNodesLinks',
    value: function setNodesLinks(nodes, links) {
      var alpha = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 1;

      this.initNodes(nodes);
      this.initLinks(links);
      this.simulation.nodes(this.nodes).force('link').links(this.links);
      var _alpha = this.simulation.alpha() + alpha;
      this.simulation.alpha(alpha > 1 ? 1 : alpha).restart();
    }
  }, {
    key: 'initDrag',
    value: function initDrag() {
      var _this4 = this;

      var event = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

      var drag = d3.drag().on('start', function (d) {
        if (!d3.event.active) {
          _this4.simulation.alphaTarget(0.5).restart();
        }
        event.start && event.start(d);
      }).on('drag', function (d) {
        d.fx = d.x = d3.event.x;
        d.fy = d.y = d3.event.y;
        event.drag && event.drag(d);
      }).on('end', function (d) {
        if (!d3.event.active) {
          _this4.simulation.alphaTarget(0);
        }
        d.fx = null;
        d.fy = null;
        event.end && event.end(d);
      });
      this.drag = drag;
    }
  }, {
    key: 'setTransform',
    value: function setTransform(transform, scale) {
      this.translate = transform;
      this.scale = scale;
    }
  }, {
    key: 'initZoom',
    value: function initZoom(event, scaleExtent) {
      var _this5 = this;

      var zoom = d3.zoom().on('start', function (d) {
        event.start && event.start(d);
      }).on('zoom', function () {
        var transform = d3.event.transform;
        var translate = [transform.x, transform.y],
            scale = transform.k;
        _this5.setTransform(translate, scale);
        event.zoom && event.zoom({ translate: translate, scale: scale });
      }).on('end', function (d) {
        event.end && event.end(d);
      });
      if (scaleExtent) {
        zoom.scaleExtent(scaleExtent);
      }
      this.zoom = zoom;
    }
  }]);
  return Simulation;
}();

exports.default = Simulation;
//# sourceMappingURL=simulation.js.map